@@ -1,21 +1,23 @@
 #[allow(deprecated, unused_imports)]
 use std::ascii::AsciiExt;
 use std::borrow::{Borrow, ToOwned};
+use std::convert::TryFrom;
 use std::env;
-use std::fs::{self, File};
-use std::io::{self, prelude::*, BufReader, IsTerminal, Read, SeekFrom};
+use std::fs;
+use std::io::{self, prelude::*, BufReader, IsTerminal, Read};
 use std::ops::Deref;
 use std::path::PathBuf;
 
 use crate::index::Indexed;
 use flate2::read::GzDecoder;
-use serde::de::{Deserialize, Deserializer, Error};
 
+use crate::read::ReverseRead;
 use crate::select::{SelectColumns, Selection};
 use crate::util;
 use crate::{CliError, CliResult};
 
-#[derive(Clone, Copy, Debug)]
+#[derive(Clone, Copy, Debug, Deserialize)]
+#[serde(try_from = "String")]
 pub struct Delimiter(pub u8);
 
 /// Delimiter represents values that can be passed from the command line that
@@ -29,10 +31,11 @@ impl Delimiter {
     }
 }
 
-impl<'de> Deserialize<'de> for Delimiter {
-    fn deserialize<D: Deserializer<'de>>(d: D) -> Result<Delimiter, D::Error> {
-        let c = String::deserialize(d)?;
-        match &*c {
+impl TryFrom<String> for Delimiter {
+    type Error = String;
+
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        match value.as_str() {
             r"\t" => Ok(Delimiter(b'\t')),
             s => {
                 if s.len() != 1 {
@@ -41,7 +44,7 @@ impl<'de> Deserialize<'de> for Delimiter {
                                        ASCII character.",
                         s
                     );
-                    return Err(D::Error::custom(msg));
+                    return Err(msg);
                 }
                 let c = s.chars().next().unwrap();
                 if c.is_ascii() {
@@ -52,65 +55,18 @@ impl<'de> Deserialize<'de> for Delimiter {
                                        to ASCII delimiter.",
                         c
                     );
-                    Err(D::Error::custom(msg))
+                    Err(msg)
                 }
             }
         }
     }
 }
 
-struct ReverseRead {
-    input: Box<File>,
-    offset: u64,
-    ptr: u64,
-}
-
-impl Read for ReverseRead {
-    fn read(&mut self, buf: &mut [u8]) -> io::Result<usize> {
-        let buff_size = buf.len() as u64;
-
-        if self.ptr == self.offset {
-            return Ok(0);
-        }
-
-        if self.offset + buff_size > self.ptr {
-            let e = (self.ptr - self.offset) as usize;
-
-            self.input.seek(SeekFrom::Start(self.offset))?;
-            self.input.read_exact(&mut buf[0..e])?;
-
-            buf[0..e].reverse();
-
-            self.ptr = self.offset;
-
-            Ok(e)
-        } else {
-            let new_position = self.ptr - buff_size;
-
-            self.input.seek(SeekFrom::Start(new_position))?;
-            self.input.read_exact(buf)?;
-            buf.reverse();
-
-            self.ptr -= buff_size;
-
-            Ok(buff_size as usize)
-        }
-    }
-}
-
-impl ReverseRead {
-    fn build(input: Box<File>, filesize: u64, offset: u64) -> ReverseRead {
-        ReverseRead {
-            input,
-            offset,
-            ptr: filesize,
-        }
-    }
-}
-
 pub trait SeekRead: Seek + Read {}
 impl<T: Seek + Read> SeekRead for T {}
 
+type PairResult = CliResult<(String, Option<String>)>;
+
 #[derive(Debug)]
 pub struct Config {
     pub path: Option<PathBuf>, // None implies <stdin>
@@ -133,12 +89,14 @@ impl Config {
             None => (None, b','),
             Some(ref s) if s.deref() == "-" => (None, b','),
             Some(ref s) => {
-                let delim = if s.ends_with(".tsv")
-                    || s.ends_with(".tab")
-                    || s.ends_with(".tsv.gz")
-                    || s.ends_with(".tab.gz")
-                {
+                let raw_s = s.strip_suffix(".gz").unwrap_or(s);
+
+                let delim = if raw_s.ends_with(".tsv") || raw_s.ends_with(".tab") {
                     b'\t'
+                } else if raw_s.ends_with(".ssv") || raw_s.ends_with(".scsv") {
+                    b';'
+                } else if raw_s.ends_with(".psv") {
+                    b'|'
                 } else {
                     b','
                 };
@@ -422,6 +380,56 @@ impl Config {
         )))
     }
 
+    pub fn pairs(
+        &self,
+        select: (&Option<SelectColumns>, &Option<SelectColumns>),
+    ) -> CliResult<Box<dyn Iterator<Item = PairResult>>> {
+        if let Some(first_sel) = &select.0 {
+            let mut csv_reader = self.reader()?;
+            let headers = csv_reader.byte_headers()?;
+            let first_column_index = first_sel.single_selection(headers, !self.no_headers)?;
+            let second_column_index_opt = select
+                .1
+                .as_ref()
+                .map(|sel| sel.single_selection(headers, !self.no_headers))
+                .transpose()?;
+
+            return Ok(Box::new(csv_reader.into_byte_records().map(
+                move |result| match result {
+                    Err(e) => Err(e)?,
+                    Ok(record) => {
+                        let a = String::from_utf8(record[first_column_index].to_vec())
+                            .expect("could not decode utf8");
+
+                        let b = second_column_index_opt.map(|second_column_index| {
+                            String::from_utf8(record[second_column_index].to_vec())
+                                .expect("could not decode utf8")
+                        });
+
+                        Ok((a, b))
+                    }
+                },
+            )));
+        }
+
+        let lines_reader = self.io_buf_reader()?;
+
+        Ok(Box::new(lines_reader.lines().filter_map(
+            |result| match result {
+                Err(e) => Some(Err(CliError::from(e))),
+                Ok(mut line) => {
+                    line.truncate(line.trim_end().len());
+
+                    if line.is_empty() {
+                        None
+                    } else {
+                        Some(Ok((line, None)))
+                    }
+                }
+            },
+        )))
+    }
+
     pub fn io_reader_for_random_access(&self) -> io::Result<Box<dyn SeekRead + 'static>> {
         let msg = "can't use provided input because it does not allow for random access (e.g. stdin or piping)".to_string();
         match self.path {
@@ -476,7 +484,7 @@ impl Config {
                 Ok(x) => match x.borrow().stream_position() {
                     Ok(_) => {
                         let filesize = x.metadata()?.len();
-                        Ok(Box::new(ReverseRead::build(Box::new(x), filesize, offset)))
+                        Ok(Box::new(ReverseRead::new(x, filesize, offset)))
                     }
                     Err(_) => Err(io::Error::new(io::ErrorKind::Unsupported, msg)),
                 },
