@@ -1,5 +1,6 @@
 use std::borrow::Cow;
 use std::cell::RefCell;
+use std::convert::TryFrom;
 use std::fs;
 use std::io;
 use std::num::NonZeroUsize;
@@ -20,7 +21,7 @@ use rand::RngCore;
 use rand_chacha::ChaCha8Rng;
 use rand_seeder::Seeder;
 use regex::{Captures, Regex};
-use serde::de::{Deserialize, DeserializeOwned, Deserializer, Error};
+use serde::de::DeserializeOwned;
 use unicode_segmentation::UnicodeSegmentation;
 use unicode_width::UnicodeWidthStr;
 
@@ -58,6 +59,7 @@ lazy_static! {
     static ref MAIN_SECTION_REGEX: Regex = Regex::new("(?m)^#+.+").unwrap();
     static ref MAIN_COMMAND_REGEX: Regex = Regex::new(r"(?m)^\s{4}[\w\-]+").unwrap();
     static ref MAIN_ALIAS_REGEX: Regex = Regex::new(r"\([^\)\s]+\)").unwrap();
+    static ref URL_REGEX: Regex = Regex::new(r"https?://\S+").unwrap();
 }
 
 pub fn colorize_help(help: &str) -> String {
@@ -74,6 +76,8 @@ pub fn colorize_help(help: &str) -> String {
         caps[0].dimmed().white().to_string()
     });
 
+    let help = URL_REGEX.replace_all(&help, |caps: &Captures| caps[0].blue().to_string());
+
     help.into_owned()
 }
 
@@ -216,7 +220,8 @@ fn create_dir_all_threadsafe(path: &Path) -> io::Result<()> {
 
 /// Represents a filename template of the form `"{}.csv"`, where `"{}"` is
 /// the splace to insert the part of the filename generated by `xan`.
-#[derive(Clone, Debug)]
+#[derive(Clone, Debug, Deserialize)]
+#[serde(try_from = "String")]
 pub struct FilenameTemplate {
     prefix: String,
     suffix: String,
@@ -259,19 +264,18 @@ impl FilenameTemplate {
     }
 }
 
-impl<'de> Deserialize<'de> for FilenameTemplate {
-    fn deserialize<D: Deserializer<'de>>(d: D) -> Result<FilenameTemplate, D::Error> {
-        let raw = String::deserialize(d)?;
-        let chunks = raw.split("{}").collect::<Vec<_>>();
+impl TryFrom<String> for FilenameTemplate {
+    type Error = &'static str;
+
+    fn try_from(value: String) -> Result<Self, Self::Error> {
+        let chunks = value.split("{}").collect::<Vec<_>>();
         if chunks.len() == 2 {
             Ok(FilenameTemplate {
                 prefix: chunks[0].to_owned(),
                 suffix: chunks[1].to_owned(),
             })
         } else {
-            Err(D::Error::custom(
-                "The --filename argument must contain one '{}'.",
-            ))
+            Err("The --filename argument must contain one '{}'.")
         }
     }
 }
